@@ -11,6 +11,7 @@
 // WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 // See the License for the specific language governing permissions and
 // limitations under the License.
+//! All edge (relationship) related.
 
 use std::ffi::{CStr, CString};
 
@@ -32,7 +33,13 @@ pub struct EdgesIterator {
 }
 
 impl EdgesIterator {
-    pub fn new(ptr: *mut mgp_edges_iterator, memgraph: &Memgraph) -> EdgesIterator {
+    pub(crate) fn new(ptr: *mut mgp_edges_iterator, memgraph: &Memgraph) -> EdgesIterator {
+        #[cfg(not(test))]
+        assert!(
+            !ptr.is_null(),
+            "Unable to create edges iterator because the given pointer is null."
+        );
+
         EdgesIterator {
             ptr,
             is_first: true,
@@ -56,27 +63,25 @@ impl Iterator for EdgesIterator {
 
     fn next(&mut self) -> Option<Edge> {
         unsafe {
-            let data: *const mgp_edge;
-            if self.is_first {
+            let data = if self.is_first {
                 self.is_first = false;
-                data = ffi::mgp_edges_iterator_get(self.ptr);
+                ffi::mgp_edges_iterator_get(self.ptr)
             } else {
-                data = ffi::mgp_edges_iterator_next(self.ptr);
-            }
+                ffi::mgp_edges_iterator_next(self.ptr)
+            };
 
             if data.is_null() {
                 None
             } else {
                 Some(match Edge::mgp_copy(data, &self.memgraph) {
                     Ok(v) => v,
-                    Err(_) => panic!("Unable to create new edge during edges iteration."),
+                    Err(_) => panic!("Unable to create edge during edges iteration."),
                 })
             }
         }
     }
 }
 
-#[derive(Debug)]
 pub struct Edge {
     ptr: *mut mgp_edge,
     memgraph: Memgraph,
@@ -93,7 +98,13 @@ impl Drop for Edge {
 }
 
 impl Edge {
-    pub fn new(ptr: *mut mgp_edge, memgraph: &Memgraph) -> Edge {
+    pub(crate) fn new(ptr: *mut mgp_edge, memgraph: &Memgraph) -> Edge {
+        #[cfg(not(test))]
+        assert!(
+            !ptr.is_null(),
+            "Unable to create edge because the given pointer is null."
+        );
+
         Edge {
             ptr,
             memgraph: memgraph.clone(),
@@ -102,16 +113,15 @@ impl Edge {
 
     /// Creates a new Edge based on [mgp_edge].
     pub(crate) unsafe fn mgp_copy(ptr: *const mgp_edge, memgraph: &Memgraph) -> MgpResult<Edge> {
-        // Test passes null ptr because nothing else is possible.
         #[cfg(not(test))]
         assert!(
             !ptr.is_null(),
-            "Unable to make edge copy because edge pointer is null."
+            "Unable to create edge copy because the given pointer is null."
         );
 
-        let mgp_copy = ffi::mgp_edge_copy(ptr, memgraph.memory());
+        let mgp_copy = ffi::mgp_edge_copy(ptr, memgraph.memory_ptr());
         if mgp_copy.is_null() {
-            return Err(MgpError::UnableToMakeEdgeCopy);
+            return Err(MgpError::UnableToCopyEdge);
         }
         Ok(Edge::new(mgp_copy, &memgraph))
     }
@@ -153,11 +163,11 @@ impl Edge {
     pub fn property(&self, name: &CStr) -> MgpResult<Property> {
         unsafe {
             let mgp_value =
-                ffi::mgp_edge_get_property(self.ptr, name.as_ptr(), self.memgraph.memory());
+                ffi::mgp_edge_get_property(self.ptr, name.as_ptr(), self.memgraph.memory_ptr());
             if mgp_value.is_null() {
                 return Err(MgpError::UnableToReturnEdgePropertyValueAllocationError);
             }
-            let value = match MgpValue::to_value(&MgpValue::new(mgp_value), &self.memgraph) {
+            let value = match MgpValue::new(mgp_value, &self.memgraph).to_value() {
                 Ok(v) => v,
                 Err(_) => return Err(MgpError::UnableToReturnEdgePropertyValueCreationError),
             };
@@ -173,7 +183,7 @@ impl Edge {
 
     pub fn properties(&self) -> MgpResult<PropertiesIterator> {
         unsafe {
-            let mgp_iterator = ffi::mgp_edge_iter_properties(self.ptr, self.memgraph.memory());
+            let mgp_iterator = ffi::mgp_edge_iter_properties(self.ptr, self.memgraph.memory_ptr());
             if mgp_iterator.is_null() {
                 return Err(MgpError::UnableToReturnEdgePropertiesIterator);
             }
