@@ -11,6 +11,7 @@
 // WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 // See the License for the specific language governing permissions and
 // limitations under the License.
+//! All related to the list datatype.
 
 use crate::memgraph::*;
 use crate::mgp::*;
@@ -21,7 +22,8 @@ use crate::value::*;
 use crate::mgp::ffi;
 use mockall_double::double;
 
-#[derive(Debug)]
+// NOTE: Not possible to implement [std::iter::IntoIterator] because the [ListIterator] holds the
+// [List] reference which needs the lifetime specifier.
 pub struct List {
     ptr: *mut mgp_list,
     memgraph: Memgraph,
@@ -46,8 +48,7 @@ impl<'a> Iterator for ListIterator<'a> {
     type Item = Value;
 
     fn next(&mut self) -> Option<Value> {
-        // TODO(gitbuda): Implement ListIterator::next using mgp primitive methods (optimal).
-        if self.position == self.list.size() {
+        if self.position >= self.list.size() {
             return None;
         }
         let value = match self.list.value_at(self.position) {
@@ -60,7 +61,13 @@ impl<'a> Iterator for ListIterator<'a> {
 }
 
 impl List {
-    pub fn new(ptr: *mut mgp_list, memgraph: &Memgraph) -> List {
+    pub(crate) fn new(ptr: *mut mgp_list, memgraph: &Memgraph) -> List {
+        #[cfg(not(test))]
+        assert!(
+            !ptr.is_null(),
+            "Unable to create list because the given pointer is null."
+        );
+
         List {
             ptr,
             memgraph: memgraph.clone(),
@@ -69,7 +76,7 @@ impl List {
 
     pub fn make_empty(capacity: u64, memgraph: &Memgraph) -> MgpResult<List> {
         unsafe {
-            let mgp_ptr = ffi::mgp_list_make_empty(capacity, memgraph.memory());
+            let mgp_ptr = ffi::mgp_list_make_empty(capacity, memgraph.memory_ptr());
             if mgp_ptr.is_null() {
                 return Err(MgpError::UnableToCreateEmptyList);
             }
@@ -79,15 +86,14 @@ impl List {
 
     /// Creates a new List based on [mgp_list].
     pub(crate) unsafe fn mgp_copy(ptr: *const mgp_list, memgraph: &Memgraph) -> MgpResult<List> {
-        // Test passes null ptr because nothing else is possible.
         #[cfg(not(test))]
         assert!(
             !ptr.is_null(),
-            "Unable to make list copy because list pointer is null."
+            "Unable to create list copy because the given pointer is null."
         );
 
         let size = ffi::mgp_list_size(ptr);
-        let mgp_copy = ffi::mgp_list_make_empty(size, memgraph.memory());
+        let mgp_copy = ffi::mgp_list_make_empty(size, memgraph.memory_ptr());
         if mgp_copy.is_null() {
             return Err(MgpError::UnableToCopyList);
         }
