@@ -11,6 +11,7 @@
 // WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 // See the License for the specific language governing permissions and
 // limitations under the License.
+//! All related to a graph path.
 
 use crate::edge::*;
 use crate::memgraph::*;
@@ -22,7 +23,6 @@ use crate::vertex::*;
 use crate::mgp::ffi;
 use mockall_double::double;
 
-#[derive(Debug)]
 pub struct Path {
     ptr: *mut mgp_path,
     memgraph: Memgraph,
@@ -38,7 +38,13 @@ impl Drop for Path {
 }
 
 impl Path {
-    pub fn new(ptr: *mut mgp_path, memgraph: &Memgraph) -> Path {
+    pub(crate) fn new(ptr: *mut mgp_path, memgraph: &Memgraph) -> Path {
+        #[cfg(not(test))]
+        assert!(
+            !ptr.is_null(),
+            "Unable to create path because the given pointer is null."
+        );
+
         Path {
             ptr,
             memgraph: memgraph.clone(),
@@ -50,16 +56,15 @@ impl Path {
         mgp_path: *const mgp_path,
         memgraph: &Memgraph,
     ) -> MgpResult<Path> {
-        // Test passes null ptr because nothing else is possible.
         #[cfg(not(test))]
         assert!(
             !mgp_path.is_null(),
-            "Unable to make path copy because path pointer is null."
+            "Unable to make path copy because the given pointer is null."
         );
 
-        let mgp_copy = ffi::mgp_path_copy(mgp_path, memgraph.memory());
+        let mgp_copy = ffi::mgp_path_copy(mgp_path, memgraph.memory_ptr());
         if mgp_copy.is_null() {
-            return Err(MgpError::UnableToMakePathCopy);
+            return Err(MgpError::UnableToCopyPath);
         }
         Ok(Path::new(mgp_copy, &memgraph))
     }
@@ -76,7 +81,7 @@ impl Path {
     /// Makes a new [Path] based on the starting [Vertex] object.
     pub fn make_with_start(vertex: &Vertex, memgraph: &Memgraph) -> MgpResult<Path> {
         unsafe {
-            let mgp_path = ffi::mgp_path_make_with_start(vertex.mgp_ptr(), memgraph.memory());
+            let mgp_path = ffi::mgp_path_make_with_start(vertex.mgp_ptr(), memgraph.memory_ptr());
             if mgp_path.is_null() {
                 return Err(MgpError::UnableToCreatePathWithStartVertex);
             }
@@ -85,7 +90,7 @@ impl Path {
     }
 
     /// Fails if the current last vertex in the path is not part of the given edge or if there is
-    /// not memory to expand the path.
+    /// no memory to expand the path.
     pub fn expand(&self, edge: &Edge) -> MgpResult<()> {
         unsafe {
             let mgp_result = ffi::mgp_path_expand(self.ptr, edge.mgp_ptr());
