@@ -11,6 +11,7 @@
 // WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 // See the License for the specific language governing permissions and
 // limitations under the License.
+//! All related to the property graph property (data key-value pair).
 
 use std::ffi::CString;
 
@@ -25,13 +26,12 @@ use mockall_double::double;
 /// All related to the property graph property (data key-value pair).
 ///
 /// Property is used in the following contexts:
-///   * return Property from PropertiesIterator
-///   * return Property directly from vertex/edge.
+///   * return Property from [PropertiesIterator]
+///   * return Property directly from [crate::vertex::Vertex] or [crate::edge::Edge].
 ///
-/// Property owns CString and Value bacause the underlying C string or value could be deleted
+/// Property owns [CString] and [Value] bacause the underlying C string or value could be deleted
 /// during the lifetime of the property. In other words, Property stores copies of underlying name
 /// and value.
-#[derive(Debug)]
 pub struct Property {
     pub name: CString,
     pub value: Value,
@@ -44,7 +44,16 @@ pub struct PropertiesIterator {
 }
 
 impl PropertiesIterator {
-    pub fn new(ptr: *mut mgp_properties_iterator, memgraph: &Memgraph) -> PropertiesIterator {
+    pub(crate) fn new(
+        ptr: *mut mgp_properties_iterator,
+        memgraph: &Memgraph,
+    ) -> PropertiesIterator {
+        #[cfg(not(test))]
+        assert!(
+            !ptr.is_null(),
+            "Unable to create properties iterator because the given pointer is null."
+        );
+
         PropertiesIterator {
             ptr,
             is_first: true,
@@ -68,13 +77,12 @@ impl Iterator for PropertiesIterator {
 
     fn next(&mut self) -> Option<Property> {
         unsafe {
-            let data: *const mgp_property;
-            if self.is_first {
+            let data = if self.is_first {
                 self.is_first = false;
-                data = ffi::mgp_properties_iterator_get(self.ptr);
+                ffi::mgp_properties_iterator_get(self.ptr)
             } else {
-                data = ffi::mgp_properties_iterator_next(self.ptr);
-            }
+                ffi::mgp_properties_iterator_next(self.ptr)
+            };
 
             if data.is_null() {
                 None
