@@ -11,12 +11,26 @@
 // WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 // See the License for the specific language governing permissions and
 // limitations under the License.
+//! Macro definitions and top level data structures.
+//!
+//! [define_procedure], [init_module], [close_module] all accept a function accepting [Memgraph]
+//! and returning [crate::result::MgpResult] because that allows using `?` operator which is a very
+//! convenient way of propagating execution errors.
+//!
+//! Example
+//!
+//! ```no run
+//! |memgraph: &Memgraph| -> MgpResult<()> {
+//!     // Implementation
+//! }
+//! ```
 
 use std::ffi::CStr;
 
 use crate::memgraph::*;
 #[double]
 use crate::mgp::ffi;
+use crate::value::*;
 use mockall_double::double;
 
 /// Defines a new procedure callable by Memgraph engine.
@@ -31,7 +45,7 @@ use mockall_double::double;
 /// Example
 ///
 /// ```no run
-/// define_procedure!(procedure_name, |memgraph: &Memgraph| -> Result<(), MgpError> {
+/// define_procedure!(procedure_name, |memgraph: &Memgraph| -> MgpResult<()> {
 ///     // Implementation
 /// }
 /// ```
@@ -53,7 +67,6 @@ macro_rules! define_procedure {
                 match $rs_func(&memgraph) {
                     Ok(_) => (),
                     Err(e) => {
-                        println!("{}", e);
                         let msg = e.to_string();
                         println!("{}", msg);
                         let c_msg =
@@ -66,21 +79,20 @@ macro_rules! define_procedure {
             panic::set_hook(prev_hook);
             match procedure_result {
                 Ok(_) => {}
-                // TODO(gitbuda): Take cause on panic and pass to mgp_result_set_error_msg.
-                // Until figuring out how to take info from panic object, set error in-place.
-                // As far as I know iterator can't return Result object and set error in-place.
                 Err(e) => {
                     println!("Procedure panic!");
+                    let memgraph = Memgraph::new(args, graph, result, memory, std::ptr::null_mut());
                     match e.downcast::<&str>() {
-                        Ok(panic_msg) => {
-                            println!("{}", panic_msg);
+                        Ok(msg) => {
+                            println!("{}", msg);
+                            let c_msg = CString::new(msg.as_bytes())
+                                .expect("Unable to create Memgraph PANIC error message!");
+                            set_memgraph_error_msg(&c_msg, &memgraph);
                         }
                         Err(_) => {
                             println!("Unknown type of panic!.");
                         }
                     }
-                    // TODO(gitbuda): Fix backtrace somehow.
-                    println!("{:?}", Backtrace::new());
                 }
             }
         }
@@ -96,7 +108,7 @@ macro_rules! define_procedure {
 ///     memgraph.add_read_procedure(
 ///         procedure_name,
 ///         c_str!("procedure_name"),
-///         &[define_type!("list", FieldType::List, FieldType::Int),],
+///         &[define_type!("list", Type::List, Type::Int),],
 ///     )?;
 ///     Ok(())
 /// });
@@ -131,6 +143,13 @@ macro_rules! init_module {
 }
 
 /// Closes Memgraph query module.
+///
+/// Example
+///
+/// ```no run
+/// close_module!(|memgraph: &Memgraph| -> MgpResult<()> {
+///     // Implementation
+/// }
 #[macro_export]
 macro_rules! close_module {
     ($close_func:expr) => {
@@ -148,8 +167,9 @@ macro_rules! close_module {
     };
 }
 
-/// Used to pass expected result field types during procedure registration.
-pub enum FieldType {
+/// Used to pass expected types (arguments, optional arguments, result field types) to the Memgraph
+/// engine.
+pub enum Type {
     Any,
     Bool,
     Number,
@@ -164,46 +184,95 @@ pub enum FieldType {
     List,
 }
 
-/// Used to pass expected result field types during procedure registration.
+/// Used to pass expected type during procedure registration for required parameters and result
+/// values.
 ///
-/// Each procedure can have multiple returning fields, each one is represented by this struct.
-/// Return type is deduced by processing types field from left to right. E.g., [FieldType::Any]
-/// means any return type, [FieldType::List], [FieldType::Int] means a list of integer values.
-pub struct ResultFieldType<'a> {
+/// The final type is deduced by processing types field from left to right. E.g., [Type::Any]
+/// means any return type, [Type::List], [Type::Int] means a list of integer values.
+///
+/// `deprecated` is used to register deprecated result type.
+pub struct NamedType<'a> {
     pub name: &'a CStr,
-    pub types: &'a [FieldType],
+    pub types: &'a [Type],
+    pub deprecated: bool,
 }
 
-/// Defines single result field type.
+/// Used to pass expected type during procedure registration for optional parameters.
 ///
-/// Example of defining the field as a list of integers.
+/// The final type is deduced by processing types field from left to right. E.g., [Type::Any]
+/// means any return type, [Type::List], [Type::Int] means a list of integer values.
+///
+/// `default` stores value in case the caller doesn't pass any value.
+pub struct OptionalNamedType<'a> {
+    pub name: &'a CStr,
+    pub types: &'a [Type],
+    pub default: &'a MgpValue,
+}
+
+/// Defines a regular type.
+///
+/// Example of defining a list of integers
 ///
 /// ```no run
-/// define_type!("name", FieldType::List, FieldType::Int);
+/// define_type!("name", Type::List, Type::Int);
 /// ```
 #[macro_export]
 macro_rules! define_type {
     ($name:literal, $($types:expr),+) => {
-        ResultFieldType {
+        NamedType {
             name: &c_str!($name),
             types: &[$($types),+],
+            deprecated: false,
         }
     };
 }
 
+/// Defines a deprecated type.
+///
+/// Example of defining a list of integers
+///
+/// ```no run
+/// define_deprecated_type!("name", Type::List, Type::Int);
+/// ```
+#[macro_export]
+macro_rules! define_deprecated_type {
+    ($name:literal, $($types:expr),+) => {
+        NamedType {
+            name: &c_str!($name),
+            types: &[$($types),+],
+            depricated: true,
+        }
+    };
+}
+
+/// Defines an optional type.
+///
+/// Example of defining an int.
+///
+/// ```no run
+/// define_optional_type!("name", &MgpValue::make_int(0, &memgraph), Type::List, Type::Int);
+/// ```
+#[macro_export]
+macro_rules! define_optional_type {
+    ($name:literal, $default:expr, $($types:expr),+) => {
+        OptionalNamedType {
+            name: &c_str!($name),
+            types: &[$($types),+],
+            default: $default,
+        }
+    };
+}
+
+/// Sets error that will be returned to the caller.
 pub fn set_memgraph_error_msg(msg: &CStr, memgraph: &Memgraph) {
     unsafe {
-        let status = ffi::mgp_result_set_error_msg(memgraph.result(), msg.as_ptr());
+        let status = ffi::mgp_result_set_error_msg(memgraph.result_ptr(), msg.as_ptr());
         if status == 0 {
             panic!("Unable to pass error message to the Memgraph engine.");
         }
     }
 }
 
-// TODO(gitbuda): Add transaction management (abort) stuff.
-// TODO(gitbuda): Deal with optional arguments.
-// TODO(gitbuda): Add support for depricated arguments.
-
 #[cfg(test)]
 mod tests {
     use c_str_macro::c_str;
