@@ -11,7 +11,9 @@
 // WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 // See the License for the specific language governing permissions and
 // limitations under the License.
+//! All related to the value (container for any data type).
 
+use std::convert::From;
 use std::ffi::{CStr, CString};
 
 use crate::edge::*;
@@ -27,16 +29,19 @@ use crate::vertex::*;
 use crate::mgp::ffi;
 use mockall_double::double;
 
+/// Creates a copy of the provided string.
+///
 /// # Safety
-/// TODO(gitbuda): Write section about safety.
+///
+/// The caller has provided a pointer that points to a valid C string. More here [CStr::from_ptr].
 pub(crate) unsafe fn create_cstring(c_char_ptr: *const i8) -> MgpResult<CString> {
     match CString::new(CStr::from_ptr(c_char_ptr).to_bytes()) {
         Ok(v) => Ok(v),
-        Err(_) => Err(MgpError::UnableToMakeCString),
+        Err(_) => Err(MgpError::UnableToCreateCString),
     }
 }
 
-// NOTE on why mutable pointer to mgp_value has to be owned by this code.
+// NOTE: on why mutable pointer to mgp_value has to be owned by this code.
 //
 // mgp_value used to return data is non-const, owned by the module code. Function to delete
 // mgp_value is non-const.
@@ -62,11 +67,14 @@ pub(crate) unsafe fn create_cstring(c_char_ptr: *const i8) -> MgpResult<CString>
 /// Useful to own `mgp_value` coming from / going into Memgraph as a result.
 ///
 /// Underlying pointer object is going to be automatically deleted.
-#[derive(Debug)]
+///
+/// NOTE: Implementing From<Value> for MgpValue is not simple because not all Value objects can
+/// contain Memgraph object (primitive types).
 pub struct MgpValue {
     // It's not wise to create a new MgpValue out of the existing value pointer because drop with a
     // valid pointer will be called multiple times -> double free problem.
     ptr: *mut mgp_value,
+    memgraph: Memgraph,
 }
 
 impl Drop for MgpValue {
@@ -80,25 +88,34 @@ impl Drop for MgpValue {
 }
 
 impl MgpValue {
-    pub fn new(ptr: *mut mgp_value) -> MgpValue {
-        MgpValue { ptr }
+    pub(crate) fn new(ptr: *mut mgp_value, memgraph: &Memgraph) -> MgpValue {
+        #[cfg(not(test))]
+        assert!(
+            !ptr.is_null(),
+            "Unable to create Memgraph value because the given pointer is null."
+        );
+
+        MgpValue {
+            ptr,
+            memgraph: memgraph.clone(),
+        }
     }
 
     pub(crate) fn mgp_ptr(&self) -> *const mgp_value {
         self.ptr
     }
 
-    pub fn to_value(&self, memgraph: &Memgraph) -> MgpResult<Value> {
-        unsafe { mgp_raw_value_to_value(self.mgp_ptr(), &memgraph) }
+    pub fn to_value(&self) -> MgpResult<Value> {
+        unsafe { mgp_raw_value_to_value(self.mgp_ptr(), &self.memgraph) }
     }
 
     pub fn make_null(memgraph: &Memgraph) -> MgpResult<MgpValue> {
         unsafe {
-            let mgp_ptr = ffi::mgp_value_make_null(memgraph.memory());
+            let mgp_ptr = ffi::mgp_value_make_null(memgraph.memory_ptr());
             if mgp_ptr.is_null() {
                 return Err(MgpError::UnableToMakeNullValue);
             }
-            Ok(MgpValue::new(mgp_ptr))
+            Ok(MgpValue::new(mgp_ptr, &memgraph))
         }
     }
 
@@ -108,11 +125,12 @@ impl MgpValue {
 
     pub fn make_bool(value: bool, memgraph: &Memgraph) -> MgpResult<MgpValue> {
         unsafe {
-            let mgp_ptr = ffi::mgp_value_make_bool(if !value { 0 } else { 1 }, memgraph.memory());
+            let mgp_ptr =
+                ffi::mgp_value_make_bool(if !value { 0 } else { 1 }, memgraph.memory_ptr());
             if mgp_ptr.is_null() {
                 return Err(MgpError::UnableToMakeBoolValue);
             }
-            Ok(MgpValue::new(mgp_ptr))
+            Ok(MgpValue::new(mgp_ptr, &memgraph))
         }
     }
 
@@ -122,11 +140,11 @@ impl MgpValue {
 
     pub fn make_int(value: i64, memgraph: &Memgraph) -> MgpResult<MgpValue> {
         unsafe {
-            let mgp_ptr = ffi::mgp_value_make_int(value, memgraph.memory());
+            let mgp_ptr = ffi::mgp_value_make_int(value, memgraph.memory_ptr());
             if mgp_ptr.is_null() {
                 return Err(MgpError::UnableToMakeIntegerValue);
             }
-            Ok(MgpValue::new(mgp_ptr))
+            Ok(MgpValue::new(mgp_ptr, &memgraph))
         }
     }
 
@@ -136,11 +154,11 @@ impl MgpValue {
 
     pub fn make_double(value: f64, memgraph: &Memgraph) -> MgpResult<MgpValue> {
         unsafe {
-            let mgp_ptr = ffi::mgp_value_make_double(value, memgraph.memory());
+            let mgp_ptr = ffi::mgp_value_make_double(value, memgraph.memory_ptr());
             if mgp_ptr.is_null() {
                 return Err(MgpError::UnableToMakeDoubleValue);
             }
-            Ok(MgpValue::new(mgp_ptr))
+            Ok(MgpValue::new(mgp_ptr, &memgraph))
         }
     }
 
@@ -150,11 +168,11 @@ impl MgpValue {
 
     pub fn make_string(value: &CStr, memgraph: &Memgraph) -> MgpResult<MgpValue> {
         unsafe {
-            let mgp_ptr = ffi::mgp_value_make_string(value.as_ptr(), memgraph.memory());
+            let mgp_ptr = ffi::mgp_value_make_string(value.as_ptr(), memgraph.memory_ptr());
             if mgp_ptr.is_null() {
                 return Err(MgpError::UnableToMakeMemgraphStringValue);
             }
-            Ok(MgpValue::new(mgp_ptr))
+            Ok(MgpValue::new(mgp_ptr, &memgraph))
         }
     }
 
@@ -162,12 +180,13 @@ impl MgpValue {
         unsafe { ffi::mgp_value_is_string(self.ptr) != 0 }
     }
 
-    /// Required because List implements Drop which destroys the underlying list object. MgpValue
-    /// takes the ownership of the list which means copy has to be created + if all goes well the
-    /// list object shouldn't be deleted.
+    /// Makes a copy of the given object returning the [MgpValue] object. [MgpValue] objects owns
+    /// the new object.
     pub fn make_list(list: &List, memgraph: &Memgraph) -> MgpResult<MgpValue> {
         unsafe {
-            let mgp_list = ffi::mgp_list_make_empty(list.size(), memgraph.memory());
+            // The new object should be manually destroyed in case something within this function
+            // fails.
+            let mgp_list = ffi::mgp_list_make_empty(list.size(), memgraph.memory_ptr());
             for item in list.iter()? {
                 let mgp_value = item.to_mgp_value(&memgraph)?;
                 if ffi::mgp_list_append(mgp_list, mgp_value.ptr) == 0 {
@@ -180,7 +199,7 @@ impl MgpValue {
                 ffi::mgp_list_destroy(mgp_list);
                 return Err(MgpError::UnableToMakeListValue);
             }
-            Ok(MgpValue::new(mgp_value))
+            Ok(MgpValue::new(mgp_value, &memgraph))
         }
     }
 
@@ -188,12 +207,13 @@ impl MgpValue {
         unsafe { ffi::mgp_value_is_list(self.ptr) != 0 }
     }
 
-    /// Required because Map implements Drop which destroys the underlying map object. MgpValue
-    /// takes the ownership of the map which means copy has to be created + if all goes well the
-    /// map object shouldn't be deleted.
+    /// Makes a copy of the given object returning the [MgpValue] object. [MgpValue] objects owns
+    /// the new object.
     pub fn make_map(map: &Map, memgraph: &Memgraph) -> MgpResult<MgpValue> {
         unsafe {
-            let mgp_map = ffi::mgp_map_make_empty(memgraph.memory());
+            // The new object should be manually destroyed in case something within this function
+            // fails.
+            let mgp_map = ffi::mgp_map_make_empty(memgraph.memory_ptr());
             for item in map.iter()? {
                 let mgp_value = match item.value.to_mgp_value(&memgraph) {
                     Ok(v) => v,
@@ -212,7 +232,7 @@ impl MgpValue {
                 ffi::mgp_map_destroy(mgp_map);
                 return Err(MgpError::UnableToMakeMapValue);
             }
-            Ok(MgpValue::new(mgp_value))
+            Ok(MgpValue::new(mgp_value, &memgraph))
         }
     }
 
@@ -220,12 +240,13 @@ impl MgpValue {
         unsafe { ffi::mgp_value_is_map(self.ptr) != 0 }
     }
 
-    /// Required because Vertex implements Drop which destroys the undrlying vertex object.
-    /// MgpValue takes the ownership of the vertex which means copy has to be created + if all goes
-    /// well the vertex object shouldn't be deleted.
+    /// Makes a copy of the given object returning the [MgpValue] object. [MgpValue] objects owns
+    /// the new object.
     pub fn make_vertex(vertex: &Vertex, memgraph: &Memgraph) -> MgpResult<MgpValue> {
         unsafe {
-            let mgp_copy = ffi::mgp_vertex_copy(vertex.mgp_ptr(), memgraph.memory());
+            // The new object should be manually destroyed in case something within this function
+            // fails.
+            let mgp_copy = ffi::mgp_vertex_copy(vertex.mgp_ptr(), memgraph.memory_ptr());
             if mgp_copy.is_null() {
                 return Err(MgpError::UnableToMakeVertexValue);
             }
@@ -234,7 +255,7 @@ impl MgpValue {
                 ffi::mgp_vertex_destroy(mgp_copy);
                 return Err(MgpError::UnableToMakeVertexValue);
             }
-            Ok(MgpValue::new(mgp_value))
+            Ok(MgpValue::new(mgp_value, &memgraph))
         }
     }
 
@@ -242,12 +263,13 @@ impl MgpValue {
         unsafe { ffi::mgp_value_is_vertex(self.ptr) != 0 }
     }
 
-    /// Required because Edge implements Drop which destroys the undrlying edge object. MgpValue
-    /// takes the ownership of the edge which means copy has to be created + if all goes well the
-    /// edge object shouldn't be deleted.
+    /// Makes a copy of the given object returning the [MgpValue] object. [MgpValue] objects owns
+    /// the new object.
     pub fn make_edge(edge: &Edge, memgraph: &Memgraph) -> MgpResult<MgpValue> {
         unsafe {
-            let mgp_copy = ffi::mgp_edge_copy(edge.mgp_ptr(), memgraph.memory());
+            // The new object should be manually destroyed in case something within this function
+            // fails.
+            let mgp_copy = ffi::mgp_edge_copy(edge.mgp_ptr(), memgraph.memory_ptr());
             if mgp_copy.is_null() {
                 return Err(MgpError::UnableToMakeEdgeValue);
             }
@@ -256,7 +278,7 @@ impl MgpValue {
                 ffi::mgp_edge_destroy(mgp_copy);
                 return Err(MgpError::UnableToMakeEdgeValue);
             }
-            Ok(MgpValue::new(mgp_value))
+            Ok(MgpValue::new(mgp_value, &memgraph))
         }
     }
 
@@ -264,12 +286,13 @@ impl MgpValue {
         unsafe { ffi::mgp_value_is_edge(self.ptr) != 0 }
     }
 
-    /// Required because Path implements Drop which destroys the undrlying path object. MgpValue
-    /// takes the ownership of the path which means copy has to be created + if all goes well the
-    /// path object shouldn't be deleted.
+    /// Makes a copy of the given object returning the [MgpValue] object. [MgpValue] objects owns
+    /// the new object.
     pub fn make_path(path: &Path, memgraph: &Memgraph) -> MgpResult<MgpValue> {
         unsafe {
-            let mgp_copy = ffi::mgp_path_copy(path.mgp_ptr(), memgraph.memory());
+            // The new object should be manually destroyed in case something within this function
+            // fails.
+            let mgp_copy = ffi::mgp_path_copy(path.mgp_ptr(), memgraph.memory_ptr());
             if mgp_copy.is_null() {
                 return Err(MgpError::UnableToMakePathValue);
             }
@@ -278,7 +301,7 @@ impl MgpValue {
                 ffi::mgp_path_destroy(mgp_copy);
                 return Err(MgpError::UnableToMakePathValue);
             }
-            Ok(MgpValue { ptr: mgp_value })
+            Ok(MgpValue::new(mgp_value, &memgraph))
         }
     }
 
@@ -290,7 +313,6 @@ impl MgpValue {
 /// Object containing/owning concrete underlying mgp objects (e.g., mgp_vertex).
 ///
 /// User code should mostly deal with these objects.
-#[derive(Debug)]
 pub enum Value {
     Null,
     Bool(bool),
@@ -321,8 +343,28 @@ impl Value {
     }
 }
 
+impl From<MgpValue> for Value {
+    fn from(item: MgpValue) -> Self {
+        match item.to_value() {
+            Ok(v) => v,
+            Err(_) => panic!("Unable to create Value from MgpValue."),
+        }
+    }
+}
+
+/// Creates copy of [mgp_value] object as a [Value] object.
+///
+/// NOTE: If would be more optimal not to copy [mgp_list], [mgp_map] and [mgp_path], but that's not
+/// possible at this point because of the way how C API iterators are implemented. E.g., after each
+/// `mgp_properties_iterator_next()` call, the previous `mgp_value` pointer shouldn't be used.
+/// There is no known way of defining the right lifetime of the returned `MgpValue` object. A
+/// solution would be to change the C API to preserve underlying values of the returned pointers
+/// during the lifetime of the Rust iterator.
+///
 /// # Safety
-/// TODO(gitbuda): Write section about safety.
+///
+/// Calls C API unsafe functions. The provided [mgp_value] object has to be a valid non-null
+/// pointer.
 pub(crate) unsafe fn mgp_raw_value_to_value(
     value: *const mgp_value,
     memgraph: &Memgraph,
@@ -361,7 +403,7 @@ pub(crate) unsafe fn mgp_raw_value_to_value(
             Ok(Value::Map(Map::mgp_copy(mgp_map, &memgraph)?))
         }
         _ => {
-            panic!("Unable to create Value object because of uncovered mgp_value type.");
+            panic!("Unable to create value object because of uncovered mgp_value type.");
         }
     }
 }
