@@ -11,6 +11,7 @@
 // WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 // See the License for the specific language governing permissions and
 // limitations under the License.
+//! All vertex (node) related.
 
 use std::ffi::{CStr, CString};
 
@@ -32,7 +33,13 @@ pub struct VerticesIterator {
 }
 
 impl VerticesIterator {
-    pub fn new(ptr: *mut mgp_vertices_iterator, memgraph: &Memgraph) -> VerticesIterator {
+    pub(crate) fn new(ptr: *mut mgp_vertices_iterator, memgraph: &Memgraph) -> VerticesIterator {
+        #[cfg(not(test))]
+        assert!(
+            !ptr.is_null(),
+            "Unable to create vertices iterator because the given pointer is null."
+        );
+
         VerticesIterator {
             ptr,
             is_first: true,
@@ -56,13 +63,12 @@ impl Iterator for VerticesIterator {
 
     fn next(&mut self) -> Option<Vertex> {
         unsafe {
-            let data: *const mgp_vertex;
-            if self.is_first {
+            let data = if self.is_first {
                 self.is_first = false;
-                data = ffi::mgp_vertices_iterator_get(self.ptr);
+                ffi::mgp_vertices_iterator_get(self.ptr)
             } else {
-                data = ffi::mgp_vertices_iterator_next(self.ptr);
-            }
+                ffi::mgp_vertices_iterator_next(self.ptr)
+            };
 
             if data.is_null() {
                 None
@@ -76,7 +82,6 @@ impl Iterator for VerticesIterator {
     }
 }
 
-#[derive(Debug)]
 pub struct Vertex {
     ptr: *mut mgp_vertex,
     memgraph: Memgraph,
@@ -93,7 +98,13 @@ impl Drop for Vertex {
 }
 
 impl Vertex {
-    pub fn new(ptr: *mut mgp_vertex, memgraph: &Memgraph) -> Vertex {
+    pub(crate) fn new(ptr: *mut mgp_vertex, memgraph: &Memgraph) -> Vertex {
+        #[cfg(not(test))]
+        assert!(
+            !ptr.is_null(),
+            "Unable to create vertex because the given pointer is null."
+        );
+
         Vertex {
             ptr,
             memgraph: memgraph.clone(),
@@ -105,16 +116,15 @@ impl Vertex {
         mgp_vertex: *const mgp_vertex,
         memgraph: &Memgraph,
     ) -> MgpResult<Vertex> {
-        // Test passes null ptr because nothing else is possible.
         #[cfg(not(test))]
         assert!(
             !mgp_vertex.is_null(),
             "Unable to make vertex copy because vertex pointer is null."
         );
 
-        let mgp_copy = ffi::mgp_vertex_copy(mgp_vertex, memgraph.memory());
+        let mgp_copy = ffi::mgp_vertex_copy(mgp_vertex, memgraph.memory_ptr());
         if mgp_copy.is_null() {
-            return Err(MgpError::UnableToMakeVertexCopy);
+            return Err(MgpError::UnableToCopyVertex);
         }
         Ok(Vertex::new(mgp_copy, &memgraph))
     }
@@ -153,12 +163,11 @@ impl Vertex {
     pub fn property(&self, name: &CStr) -> MgpResult<Property> {
         unsafe {
             let mgp_value =
-                ffi::mgp_vertex_get_property(self.ptr, name.as_ptr(), self.memgraph.memory());
+                ffi::mgp_vertex_get_property(self.ptr, name.as_ptr(), self.memgraph.memory_ptr());
             if mgp_value.is_null() {
                 return Err(MgpError::UnableToGetVertexProperty);
             }
-            // TODO(gitbuda): Figure out how to pass Snafu memgraph here.
-            let value = MgpValue::to_value(&MgpValue::new(mgp_value), &self.memgraph)?;
+            let value = MgpValue::new(mgp_value, &self.memgraph).to_value()?;
             match CString::new(name.to_bytes()) {
                 Ok(c_string) => Ok(Property {
                     name: c_string,
@@ -171,7 +180,8 @@ impl Vertex {
 
     pub fn properties(&self) -> MgpResult<PropertiesIterator> {
         unsafe {
-            let mgp_iterator = ffi::mgp_vertex_iter_properties(self.ptr, self.memgraph.memory());
+            let mgp_iterator =
+                ffi::mgp_vertex_iter_properties(self.ptr, self.memgraph.memory_ptr());
             if mgp_iterator.is_null() {
                 return Err(MgpError::UnableToReturnVertexPropertiesIterator);
             }
@@ -181,7 +191,7 @@ impl Vertex {
 
     pub fn in_edges(&self) -> MgpResult<EdgesIterator> {
         unsafe {
-            let mgp_iterator = ffi::mgp_vertex_iter_in_edges(self.ptr, self.memgraph.memory());
+            let mgp_iterator = ffi::mgp_vertex_iter_in_edges(self.ptr, self.memgraph.memory_ptr());
             if mgp_iterator.is_null() {
                 return Err(MgpError::UnableToReturnVertexInEdgesIterator);
             }
@@ -191,7 +201,7 @@ impl Vertex {
 
     pub fn out_edges(&self) -> MgpResult<EdgesIterator> {
         unsafe {
-            let mgp_iterator = ffi::mgp_vertex_iter_out_edges(self.ptr, self.memgraph.memory());
+            let mgp_iterator = ffi::mgp_vertex_iter_out_edges(self.ptr, self.memgraph.memory_ptr());
             if mgp_iterator.is_null() {
                 return Err(MgpError::UnableToReturnVertexOutEdgesIterator);
             }
